/******************************************************************************/
/****                                Tables                                ****/
/******************************************************************************/


CREATE GENERATOR WWW_BANERY_ID_GEN;

CREATE TABLE WWW_BANERY (
    ID                INTEGER NOT NULL,
    NAZWA             VARCHAR(16) NOT NULL,
    URL               VARCHAR(128),
    PUBLIKACJA_OD     DATE,
    PUBLIKACJA_DO     DATE,
    ODSLONY           INTEGER,
    ODSLONY_LIMIT     INTEGER,
    ODSLONY_DZIENNIE  SMALLINT,
    ODSLONY_UU        SMALLINT,
    KLIKNIECIA        INTEGER,
    TYP               VARCHAR(16),
    PLIK              VARCHAR(128) NOT NULL,
    POZYCJA           VARCHAR(32) NOT NULL,
    TARGET            SMALLINT
);




/******************************************************************************/
/****                             Primary Keys                             ****/
/******************************************************************************/

ALTER TABLE WWW_BANERY ADD CONSTRAINT PK_WWW_BANERY PRIMARY KEY (ID);


/******************************************************************************/
/****                               Triggers                               ****/
/******************************************************************************/


SET TERM ^ ;


/******************************************************************************/
/****                         Triggers for tables                          ****/
/******************************************************************************/



/* Trigger: WWW_BANERY_ID_TRIG */
CREATE TRIGGER WWW_BANERY_ID_TRIG FOR WWW_BANERY
ACTIVE BEFORE INSERT POSITION 0
AS
begin

  IF (NEW.ID IS NULL) THEN
    NEW.ID = GEN_ID(WWW_BANERY_ID_GEN,1);

end
^


SET TERM ; ^



/* Fields descriptions */

DESCRIBE FIELD ODSLONY_LIMIT TABLE WWW_BANERY
'ilosc zaplanowanych odslon';

DESCRIBE FIELD ODSLONY_DZIENNIE TABLE WWW_BANERY
'1 - rozklada ODSLONY_LIMIT na dni okreslone w PUBLIKACJA_OD - PUBLIKACJA_DO, co daje dzienny limit odslon';

DESCRIBE FIELD ODSLONY_UU TABLE WWW_BANERY
'Zlicza tylko odslony unikalnych uzytkownikow (zapis w ciastkach)';

DESCRIBE FIELD TARGET TABLE WWW_BANERY
'1 - w nowym oknie, 0 - w tym samym';



/******************************************************************************/
/****                              Privileges                              ****/
/******************************************************************************/


/* Privileges of roles */
GRANT ALL ON WWW_BANERY TO WWW_ROLE;




